import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import hash from '@emotion/hash';
import useGlobalCache from './useGlobalCache';
import { token2key } from './util';
/**
 * Cache theme derivative token as global shared one
 * @param theme Theme entity
 * @param tokens List of tokens, used for cache. Please do not dynamic generate object directly
 * @param option Additional config
 * @returns Call Theme.getDerivativeToken(tokenObject) to get token
 */

export default function useCacheToken(theme, tokens) {
  var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var _option$salt = option.salt,
      salt = _option$salt === void 0 ? '' : _option$salt;
  var tokenStr = token2key(Object.assign.apply(Object, [{}].concat(_toConsumableArray(tokens))));
  var cachedToken = useGlobalCache('token', [theme.id, salt, tokenStr], function () {
    var mergedDesignToken = Object.assign.apply(Object, [{}].concat(_toConsumableArray(tokens)));
    var derivativeToken = theme.getDerivativeToken(mergedDesignToken); // Optimize for `useStyleRegister` performance

    var tokenKey = "".concat(salt, "_").concat(token2key(derivativeToken));
    derivativeToken._tokenKey = tokenKey;
    var hashId = "css-".concat(hash(tokenKey));
    derivativeToken._hashId = hashId;
    return [derivativeToken, hashId];
  });
  return cachedToken;
}