import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import * as React from 'react';
import { updateCSS, removeCSS } from "rc-util/es/Dom/dynamicCSS";
import canUseDom from "rc-util/es/Dom/canUseDom";
import hash from '@emotion/hash'; // @ts-ignore

import unitless from '@emotion/unitless';
import { compile, serialize, stringify } from 'stylis';
import useGlobalCache from './useGlobalCache';
import CacheContext from './CacheContext';
import { token2key } from './util';
var isClientSide = canUseDom(); // ============================================================================
// ==                                 Parser                                 ==
// ============================================================================
// Preprocessor style content to browser support one

function normalizeStyle(styleStr) {
  return serialize(compile(styleStr), stringify);
} // Parse CSSObject to style content


export var parseStyle = function parseStyle(interpolation, hashId) {
  var root = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var styleStr = '';

  function flattenList(list) {
    var fullList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    list.forEach(function (item) {
      if (Array.isArray(item)) {
        flattenList(item, fullList);
      } else if (item) {
        fullList.push(item);
      }
    });
    return fullList;
  }

  var flattenStyleList = flattenList(Array.isArray(interpolation) ? interpolation : [interpolation]);
  flattenStyleList.forEach(function (style) {
    if (style._keyframe) {
      // Keyframe
      var keyframe = style;
      styleStr += "@keyframes ".concat(keyframe.getName(hashId)).concat(parseStyle(keyframe.style, hashId, false));
    } else {
      // Normal CSSObject
      Object.keys(style).forEach(function (key) {
        var value = style[key];

        if (_typeof(value) === 'object' && value) {
          // 当成嵌套对象来处理
          var mergedKey = root && hashId ? ".".concat(hashId).concat(key) : key;
          styleStr += "".concat(mergedKey).concat(parseStyle(value, hashId, false));
        } else {
          // 如果是样式则直接插入
          var styleName = key.replace(/[A-Z]/g, function (match) {
            return "-".concat(match.toLowerCase());
          }); // Auto suffix with px

          var formatValue = value;

          if (!unitless[key] && typeof formatValue === 'number' && formatValue !== 0) {
            formatValue = "".concat(formatValue, "px");
          }

          styleStr += "".concat(styleName, ":").concat(formatValue, ";");
        }
      });
    }
  });

  if (!root) {
    styleStr = "{".concat(styleStr, "}");
  }

  return styleStr;
}; // ============================================================================
// ==                                Register                                ==
// ============================================================================
// We takes `theme.id` as part of hash to avoid HMR hit to remove the same style

function uniqueHash(path, styleStr) {
  return hash("".concat(path.join('%')).concat(styleStr));
}
/**
 * Register a style to the global style sheet.
 */


export default function useStyleRegister(info, styleFn) {
  var theme = info.theme,
      token = info.token,
      path = info.path,
      hashId = info.hashId;

  var _React$useContext = React.useContext(CacheContext),
      autoClear = _React$useContext.autoClear;

  var tokenKey = token._tokenKey || token2key(token);
  var fullPath = [theme.id, tokenKey].concat(_toConsumableArray(path));
  useGlobalCache('style', fullPath, // Create cache if needed
  function () {
    var styleObj = styleFn();
    var styleStr = normalizeStyle(parseStyle(styleObj, hashId));
    var styleId = uniqueHash(fullPath, styleStr);

    if (isClientSide) {
      updateCSS(styleStr, styleId);
    }

    return styleStr;
  }, // Remove cache if no need
  function (styleStr) {
    if (autoClear && isClientSide) {
      var styleId = uniqueHash(fullPath, styleStr);
      removeCSS(styleId);
    }
  });
}