"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCacheToken;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _hash = _interopRequireDefault(require("@emotion/hash"));

var _useGlobalCache = _interopRequireDefault(require("./useGlobalCache"));

var _util = require("./util");

/**
 * Cache theme derivative token as global shared one
 * @param theme Theme entity
 * @param tokens List of tokens, used for cache. Please do not dynamic generate object directly
 * @param option Additional config
 * @returns Call Theme.getDerivativeToken(tokenObject) to get token
 */
function useCacheToken(theme, tokens) {
  var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var _option$salt = option.salt,
      salt = _option$salt === void 0 ? '' : _option$salt;
  var tokenStr = (0, _util.token2key)(Object.assign.apply(Object, [{}].concat((0, _toConsumableArray2.default)(tokens))));
  var cachedToken = (0, _useGlobalCache.default)('token', [theme.id, salt, tokenStr], function () {
    var mergedDesignToken = Object.assign.apply(Object, [{}].concat((0, _toConsumableArray2.default)(tokens)));
    var derivativeToken = theme.getDerivativeToken(mergedDesignToken); // Optimize for `useStyleRegister` performance

    var tokenKey = "".concat(salt, "_").concat((0, _util.token2key)(derivativeToken));
    derivativeToken._tokenKey = tokenKey;
    var hashId = "css-".concat((0, _hash.default)(tokenKey));
    derivativeToken._hashId = hashId;
    return [derivativeToken, hashId];
  });
  return cachedToken;
}