"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useStyleRegister;
exports.parseStyle = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var React = _interopRequireWildcard(require("react"));

var _dynamicCSS = require("rc-util/lib/Dom/dynamicCSS");

var _canUseDom = _interopRequireDefault(require("rc-util/lib/Dom/canUseDom"));

var _hash = _interopRequireDefault(require("@emotion/hash"));

var _unitless = _interopRequireDefault(require("@emotion/unitless"));

var _stylis = require("stylis");

var _useGlobalCache = _interopRequireDefault(require("./useGlobalCache"));

var _CacheContext = _interopRequireDefault(require("./CacheContext"));

var _util = require("./util");

// @ts-ignore
var isClientSide = (0, _canUseDom.default)(); // ============================================================================
// ==                                 Parser                                 ==
// ============================================================================
// Preprocessor style content to browser support one

function normalizeStyle(styleStr) {
  return (0, _stylis.serialize)((0, _stylis.compile)(styleStr), _stylis.stringify);
} // Parse CSSObject to style content


var parseStyle = function parseStyle(interpolation, hashId) {
  var root = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var styleStr = '';

  function flattenList(list) {
    var fullList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    list.forEach(function (item) {
      if (Array.isArray(item)) {
        flattenList(item, fullList);
      } else if (item) {
        fullList.push(item);
      }
    });
    return fullList;
  }

  var flattenStyleList = flattenList(Array.isArray(interpolation) ? interpolation : [interpolation]);
  flattenStyleList.forEach(function (style) {
    if (style._keyframe) {
      // Keyframe
      var keyframe = style;
      styleStr += "@keyframes ".concat(keyframe.getName(hashId)).concat(parseStyle(keyframe.style, hashId, false));
    } else {
      // Normal CSSObject
      Object.keys(style).forEach(function (key) {
        var value = style[key];

        if ((0, _typeof2.default)(value) === 'object' && value) {
          // 当成嵌套对象来处理
          var mergedKey = root && hashId ? ".".concat(hashId).concat(key) : key;
          styleStr += "".concat(mergedKey).concat(parseStyle(value, hashId, false));
        } else {
          // 如果是样式则直接插入
          var styleName = key.replace(/[A-Z]/g, function (match) {
            return "-".concat(match.toLowerCase());
          }); // Auto suffix with px

          var formatValue = value;

          if (!_unitless.default[key] && typeof formatValue === 'number' && formatValue !== 0) {
            formatValue = "".concat(formatValue, "px");
          }

          styleStr += "".concat(styleName, ":").concat(formatValue, ";");
        }
      });
    }
  });

  if (!root) {
    styleStr = "{".concat(styleStr, "}");
  }

  return styleStr;
}; // ============================================================================
// ==                                Register                                ==
// ============================================================================
// We takes `theme.id` as part of hash to avoid HMR hit to remove the same style


exports.parseStyle = parseStyle;

function uniqueHash(path, styleStr) {
  return (0, _hash.default)("".concat(path.join('%')).concat(styleStr));
}
/**
 * Register a style to the global style sheet.
 */


function useStyleRegister(info, styleFn) {
  var theme = info.theme,
      token = info.token,
      path = info.path,
      hashId = info.hashId;

  var _React$useContext = React.useContext(_CacheContext.default),
      autoClear = _React$useContext.autoClear;

  var tokenKey = token._tokenKey || (0, _util.token2key)(token);
  var fullPath = [theme.id, tokenKey].concat((0, _toConsumableArray2.default)(path));
  (0, _useGlobalCache.default)('style', fullPath, // Create cache if needed
  function () {
    var styleObj = styleFn();
    var styleStr = normalizeStyle(parseStyle(styleObj, hashId));
    var styleId = uniqueHash(fullPath, styleStr);

    if (isClientSide) {
      (0, _dynamicCSS.updateCSS)(styleStr, styleId);
    }

    return styleStr;
  }, // Remove cache if no need
  function (styleStr) {
    if (autoClear && isClientSide) {
      var styleId = uniqueHash(fullPath, styleStr);
      (0, _dynamicCSS.removeCSS)(styleId);
    }
  });
}